Rice.require 'Native'
Rice.require 'Bit'
Rice.require 'Rice.PrinterDriver'
Rice.require 'Rice.CapsDB'
Rice.require 'Rice.LangDB'
Rice.require 'Rice.ExtensionInfo'
Rice.require 'Rice.Window'
Rice.require 'exstring'
Rice.require 'FileDialog'
Rice.require 'CDBTableItem'
Rice.require 'UTF8'

local cp_acp = 0

-----------------------------------------
local filetype_txt = (function()
  local txt = Rice.LangDB.get('driverinfo_file')
  if txt then
    return UTF8.decode(txt, cp_acp)
  else
    return 'Driver Information File(*.txt)'
  end
end)()

local file_path = FileDialog.save(
  'txt',
  filetype_txt,
  Bit.OR(
    FileDialog.OFN_PATHMUSTEXIST,
    FileDialog.OFN_HIDEREADONLY,
    FileDialog.OFN_NONETWORKBUTTON,
    FileDialog.OFN_OVERWRITEPROMPT
  ),
  0, --CSIDL_DESKTOP
  'driver_info.txt'
)

if(not file_path) then
  return true
end

-----------------------------------------

local info = {}

function info:section(name)
  table.insert(self, '[' .. name .. ']')
  table.insert(self, '')
  return self
end
  
function info:add(line)
  table.insert(self, line)
  return self
end

-----------------------------------------

local function get_osver()
  local OSVERSIONINFOEX = Native.Struct()
  :dword('dwOSVersionInfoSize')
  :dword('dwMajorVersion')
  :dword('dwMinorVersion')
  :dword('dwBuildNumber')
  :dword('dwPlatformId')
  :byte_array('szCSDVersion', 128)
  :word('wServicePackMajor')
  :word('wServicePackMinor')
  :word('wSuiteMask')
  :byte('wProductType')
  :byte('wReserved')
  :pack()
  
  local osver = OSVERSIONINFOEX:instance()
  osver:set_dword('dwOSVersionInfoSize', OSVERSIONINFOEX:size())

  Native.Dll('kernel32.dll')
  :api('GetVersionEx')
  :with_ptr(osver:memory():ptr())
  :call_int()

  return osver
end
  
-----------------------------------------
  
local cdb = Rice.CapsDB
 
local function lang(id)
  local str = Rice.LangDB.get(id)
  return (type(str) == 'string') and str:match('^([^:]+)') or nil
end

-----------------------------------------
  
local function add_items(info, items, filter)

  local function id_word(id, word)
    return string.format('%s <%s>', id, word)
  end

  local add_value = {
    pickone = function(cur, item)
      local cur_str = lang(item .. '_' .. cur) or lang(cur) or ''
      info:add('  ' .. id_word(cur, cur_str))        
    end,

    range = function(cur, item, detail)
      local dec = detail.decimalnum
      info:add('  ' .. id_word(cur, tonumber(cur)
        and string.format(
          ((dec == 0) and '%d' or ('%.' .. dec .. 'f')) ,
          tonumber(cur) / math.pow(10, dec))
        or ''))
    end,

    string = function(cur)
      info:add('  "' .. cur .. '"')
    end,

    table = function (cur, item, detail)
      local columns = detail.columns
      local tbl = CDBTableItem.decode_table(cur)
      for i, row in ipairs(tbl) do
        local row_str = ''
        for j, elm in ipairs(row) do
          local elm_str = lang(columns[j] .. '_' .. elm) or lang(elm) or elm
          row_str = row_str .. id_word(elm, elm_str) .. ', '          
        end
        info:add('  ' .. row_str)
      end
    end,
      
    attribute = function(cur)
      info:add('  ' .. cur)
    end
  }
    
  for i, item in ipairs(items) do
    if(not filter or filter(item)) then
    
      local item_str = lang(item) or ''
      local detail = cdb.get_detail(item);
      local cur = detail.current or '';
      
      info:add(id_word(item, item_str) .. ' :')
      if(add_value[detail.type]) then
        add_value[detail.type](cur, item, detail)
      end
      
      info:add('')
    end
  end
end
  
-----------------------------------------

local osver = get_osver()
info:section('OS Version')
:add('OSVERSIONINFOEX')
:add('  dwMajorVersion : ' .. osver:get_dword('dwMajorVersion'))
:add('  dwMinorVersion : ' .. osver:get_dword('dwMinorVersion'))
:add('  dwBuildNumber : ' .. osver:get_dword('dwBuildNumber'))
:add('  dwPlatformId : ' .. osver:get_dword('dwPlatformId'))
:add('  szCSDVersio : ' .. osver:get_string('szCSDVersion'))
:add('  wServicePackMajor : ' .. osver:get_word('wServicePackMajor'))
:add('  wServicePackMinor : ' .. osver:get_word('wServicePackMinor'))
:add('  wSuiteMask : ' .. string.format('0x%08x', osver:get_word('wSuiteMask')))
:add('  wProductType : ' .. osver:get_byte('wProductType'))
:add('')

info:section('Driver Configuration')
:add('DriverVersion : ' .. Rice.PrinterDriver.get_printer_data('common_scope', 'DriverProductVersion'))
:add('')
:add('Platform : ' .. Native.platform())
:add('')
local drv_conf = Rice.PrinterDriver.configuration()
for component ,ver in pairs(drv_conf) do
  info:add(component:match('\\([^\\]+)$'))
  if(ver.file_version) then
    info:add('  ' .. 'FileVersion : ' ..  ver.file_version)
  end    
  info:add('  ' .. 'Date : ' ..  ver.date):add('')
end

info:section('Options')
add_items(info, cdb.options())

info:section('Features')
add_items(info, cdb.features(), function(item) return not item:find('btn') end)

info:section('Extensions')
local exts = Rice.ExtensionInfo.names()
for i, ext in ipairs(exts) do
  info:add(string.format('%s <%s>', ext, lang(ext) or ""))
  local ext_info = Rice.ExtensionInfo.get(ext)
  for k, v in pairs(ext_info) do
    info:add('  ' .. k .. ' : ' .. v)
  end
end

----------------------------------------

local file = io.open(file_path, 'w')
if(file) then
  file:write(table.concat(info, '\r\n'))
  file:close()
end
  
return true
