class 'BigInteger'

local BASE  = 10000
local BASEW = 4

local function is_zero(self)
  return (#self.digit == 1 and self.digit[1] == 0)
end

local function shorten(self)
  local w = #self.digit
  for i = w, 2, -1 do
    if self.digit[i] == 0 then
      self.digit[i] = nil
    else
      break
    end
  end
  return self
end

local error_invalid_initializer = 'invalid initializer'
local error_zero_division = 'divided by zero'

local function zero_divide()
  local ret = BigInteger()
  ret.error = error_zero_division
  return ret
end

------------------------------------------------------------
-- consrructor
------------------------------------------------------------
function BigInteger:__init(num)

  if not num then
    self.error = error_invalid_initializer
    return
  end
  
  local ty = type(num)
  if ty == 'string' then
    local s, n = num:match('^(-?)(%d+)$')
    if n then
      self.digit = {0}
      self.sign = not (s == '-')
      local length = #n
      for i = 1, math.ceil(length / BASEW) do
        self.digit[i] = tonumber(n:sub(
          math.max(length - i * BASEW + 1, 0),
          length - (i - 1) * BASEW))
      end
      shorten(self)
      if is_zero(self) then self.sign = true end
      return
    end
    
  elseif ty == 'number' then
    self.digit = {0}
    self.sign = (num >= 0)
    num = math.floor(math.abs(num))
    local i = 1
    while(num ~= 0) do
      self.digit[i] = num % BASE
      num = math.floor(num / BASE)
      i = i + 1
    end
    return
    
  elseif ty == 'userdata' and getmetatable(self) == getmetatable(num) then
    self.digit = {0}
    self.sign = num.sign
    for i, v in ipairs(num.digit) do
      self.digit[i] = v
    end
    return
  end

  self.error = error_invalid_initializer
end

------------------------------------------------------------
-- query whether it's valid integer
------------------------------------------------------------
function BigInteger:valid()
  return (self and self.digit)
end

------------------------------------------------------------
-- tostring
------------------------------------------------------------
function BigInteger:__tostring()
  if not self:valid() then return self.error end
  local str = ''
  for i, v in ipairs(self.digit) do
    str = ('%04d'):format(v) .. str
  end
  str = str:match('^0*([^0].*)$') or '0'
  return (self.sign) and str or '-' .. str
end

local EQUAL = 0
local LEFT  = 1
local RIGHT = 2

local function comp_abs(left, right)
  local w = math.max(#left.digit, #right.digit)
  for i = w, 1, -1 do
    if (left.digit[i] or 0) < (right.digit[i] or 0) then
      return RIGHT
    elseif (left.digit[i] or 0) > (right.digit[i] or 0) then
      return LEFT
    end
  end
  return EQUAL 
end

------------------------------------------------------------
-- operator <
------------------------------------------------------------
function BigInteger:__lt(other)
  if not self:valid() or not other:valid() then return nil end
  if self.sign ~= other.sign then return other.sign end
  local larger = comp_abs(self, other)
  return (
    (larger == RIGHT and self.sign) or
    (larger == LEFT and not self.sign)
  )
end

------------------------------------------------------------
-- operator <=
------------------------------------------------------------
function BigInteger:__le(other)
  if not self:valid() or not other:valid() then return nil end
  if self.sign ~= other.sign then return other.sign end
  local larger = comp_abs(self, other)
  return (
    larger == EQUAL or
    (larger == RIGHT and self.sign) or
    (larger == LEFT and not self.sign)
  )
end

------------------------------------------------------------
-- operator ==
------------------------------------------------------------
function BigInteger:__eq(other)
  if not self:valid() or not other:valid() then return nil end
  if self.sign ~= other.sign then return false end
  return (EQUAL == comp_abs(self, other))
end

local function move_up(self)
  local w = #self.digit
  for i = 1, w do
    if self.digit[i] >= BASE then
      local carry = math.floor(self.digit[i] / BASE)
      self.digit[i] = self.digit[i] % BASE
      self.digit[i + 1] = (self.digit[i + 1] or 0) + carry
    end
  end
  return self
end

local function move_down(self)
  local w = #self.digit
  for i = 1, w - 1 do
    if self.digit[i] < 0 then 
      local carry = math.floor(self.digit[i] / BASE)
      self.digit[i] = self.digit[i] - carry * BASE
      self.digit[i + 1] = self.digit[i + 1] + carry
    end
  end
  return shorten(self)
end

local function add(left, right)
  local ret = BigInteger(0)
  local w = math.max(#left.digit, #right.digit)
  for i = 1, w do
    ret.digit[i] = (left.digit[i] or 0 ) + (right.digit[i] or 0)
  end
  ret.sign = left.sign
  return move_up(ret)
end

local function sub(left, right, sign)
  local ret = BigInteger(0)
  local w = math.max(#left.digit, #right.digit)
  for i = 1, w do
    ret.digit[i] = (left.digit[i] or 0 ) - (right.digit[i] or 0)
  end
  ret.sign = left.sign
  return move_down(ret)
end

local function add_sub(self, other, add_condition)
  if add_condition then
    return add(self, other)
  else
    return ({
      [EQUAL] = function() return BigInteger(0) end,
      [LEFT]  = function() return sub(self, other) end,
      [RIGHT] = function() return sub(other, self) end
    })[comp_abs(self, other)]()
  end  
end

------------------------------------------------------------
-- operator +
------------------------------------------------------------
function BigInteger:__add(other)
  if not self:valid() then return self end
  if not other:valid() then return other end

  return add_sub(self, other, self.sign == other.sign)
end

------------------------------------------------------------
-- operator -
------------------------------------------------------------
function BigInteger:__sub(other)
  if not self:valid() then return self end
  if not other:valid() then return other end

  return add_sub(self, other, self.sign ~= other.sign)
end

------------------------------------------------------------
-- operator *
------------------------------------------------------------
function BigInteger:__mul(other)
  if not self:valid() then return self end
  if not other:valid() then return other end
  
  local ret = BigInteger(0)
  if is_zero(self) or is_zero(other) then return ret end
  ret.sign = (self.sign == other.sign)
  for i, x in ipairs(self.digit) do
    for j, y  in ipairs(other.digit) do
      ret.digit[i + j - 1] = (ret.digit[i + j - 1] or 0) + x * y
    end
    move_up(ret)
  end
  return ret
end

------------------------------------------------------------
-- operator /
------------------------------------------------------------
function BigInteger:__div(other)
  if not self:valid() then return self end
  if not other:valid() then return other end
  if is_zero(other) then return zero_divide() end
  
  local larger = comp_abs(self, other)
  local sign = (self.sign == other.sign)
  if is_zero(self) or larger == RIGHT then return BigInteger(0) 
  elseif larger == EQUAL then return sign and BigInteger(1) or BigInteger(-1)  
  end

  local ret = BigInteger(0)
  ret.sign = sign

  local factor = BigInteger(math.floor(BASE / (other.digit[#other.digit] + 1)))

  local numer = BigInteger(self) * factor
  local denom = BigInteger(other) * factor
  local denomsize = #denom.digit
  
  local i = #numer.digit
  for j = #numer.digit - denomsize + 1, 1, -1 do
    ret.digit[j] = (numer.digit[i + 1] or 0) * BASE + numer.digit[i]
    ret.digit[j] = math.floor(ret.digit[j] / denom.digit[denomsize])
    local t = BigInteger(0)
    for k = 1, j + denomsize do t.digit[k] = 0 end
    for k = 1 , denomsize do
      t.digit[j + k - 1] = ret.digit[j] * denom.digit[k] 
    end
    move_up(t)
    while(numer < t) do
      ret.digit[j] = ret.digit[j] - 1
      for k = 1 , denomsize do
        t.digit[j + k - 1] = t.digit[j + k - 1] -  denom.digit[k]
      end
      move_down(t)
    end
    numer = numer - t
    i = i - 1 
  end
  return shorten(ret)
end

